#include <iostream>
#include "executor/threaded/ThreadedExecutor.h"
#include "executor/select/SelectExecutor.h"
#include "executor/poll/PollExecutor.h"
#include "executor/epoll/EPollExecutor.h"

#include <thread>
#include <gflags/gflags.h>

DEFINE_string(ip, "127.0.0.1", "IP address to connect to");
DEFINE_int32(port, 7777, "Port to connect to");
DEFINE_int32(testWaitSeconds, 30, "Time to wait between tests (s)");
DEFINE_string(delays, "10,50,100,250,500,1000,1500,2000,4000,10000", "delay length (comma separated)");
DEFINE_string(connections, "1,2,5,10,20,40,50,100", "concurrent connections (comma separated)");
DEFINE_bool(select, false, "Do Select Test");
DEFINE_bool(poll, false, "Do poll Test");
DEFINE_bool(epoll, false, "Do epoll Test");
DEFINE_bool(threaded, false, "Do threaded Test");
DEFINE_bool(all, true, "Do All Test");

void runTest(TestExecutor *test) {
    test->execute();
    delete test;
    std::cout << "Sleep " << FLAGS_testWaitSeconds << " seconds for next test" << std::endl;
    std::this_thread::sleep_for(std::chrono::seconds(FLAGS_testWaitSeconds));
}

std::vector<int> parseCSV(std::string value) {
    std::vector<int> values;

    bool finished = false;

    while (!finished) {
        ssize_t index = value.find(",");
        std::string to_process;
        if (index == std::string::npos) {
            // EVERYTHING is a number
            to_process = value;
            finished = true;
        } else {
            to_process = value.substr(0, index);
            value = value.substr(index + 1);
        }

        int val = std::stoi(to_process);
        values.push_back(val);
    }

    return values;
}

int main(int argc, char *argv[]) {
    gflags::ParseCommandLineFlags(&argc, &argv, true);

    std::vector<int> conns = parseCSV(FLAGS_connections);
    std::vector<int> delays = parseCSV(FLAGS_delays);

    bool reuse[] = {true};

    bool threaded = FLAGS_all || FLAGS_threaded;
    bool select = FLAGS_all || FLAGS_select;
    bool poll = FLAGS_all || FLAGS_poll;
    bool epoll = FLAGS_all || FLAGS_epoll;

    for (int delay : delays) {
        for (int threadCount : conns) {
            for (bool reusing : reuse) {
                if (threaded) {
                    auto *threadedExecutor = new ThreadedExecutor(FLAGS_ip, FLAGS_port, threadCount, delay, reusing);
                    runTest(threadedExecutor);
                }

                if (select) {
                    auto *selectExecutor = new SelectExecutor(FLAGS_ip, FLAGS_port, threadCount, delay, reusing);
                    runTest(selectExecutor);
                }

                if (poll) {
                    auto *pollExecutor = new PollExecutor(FLAGS_ip, FLAGS_port, threadCount, delay, reusing);
                    runTest(pollExecutor);
                }

                if (epoll) {
                    auto *epollExecutor = new EPollExecutor(FLAGS_ip, FLAGS_port, threadCount, delay, reusing);
                    runTest(epollExecutor);
                }
            }
        }
    }
    return 0;
}